# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function, unicode_literals

import datetime
import argparse
from .. import create_searcher

def info(string):
    print(string)

def getresults(args):
    kw = dict(delimiter=args.delimiter, quotechar=args.quotechar, lower=args._from, upper=args.to)
    searcher = create_searcher(args.filepath, args.start, args.end, **kw)
    with searcher as result:
        for row in result:
            yield row

def search(args):
    info('## in {file} from {fromd} to {to}'.format(file=args.filepath, fromd=args._from, to=args.to))
    for row in getresults(args):
        info(args.delimiter.join(row))

def validatedatetime(args):
    try:
        fromd = datetime.datetime.strptime(args._from, '%Y-%m-%d %H:%M:%S')
    except ValueError:
        raise ValueError('wrong datetime format {}'.format(args._from))

    if args.to is None:
        args.to = (fromd + datetime.timedelta(minutes=1)).strftime('%Y-%m-%d %H:%M:%S')
    try:
        datetime.datetime.strptime(args.to, '%Y-%m-%d %H:%M:%S')
    except ValueError:
        raise ValueError('wrong datetime format {}'.format(args._from))

    if args.prefix:
        args.to = args.prefix + args.to
        args._from = args.prefix + args._from

def main():
    parser = argparse.ArgumentParser(description='search bz2 accesslog')
    parser.add_argument('-s', '--start', default=1, type=int, help='start collumn')
    parser.add_argument('-e', '--end', default=2, type=int, help='end collumn')
    parser.add_argument('-f', '--from', dest='_from', help="From time for log reading / '%%Y-%%m-%%d %%H:%%M:%%S'n")
    parser.add_argument('-t', '--to', help="To time for log reading / '%%Y-%%m-%%d %%H:%%M:%%S'")
    parser.add_argument('-d', '--delimiter', help="log dalimiter", default=b" ", type=bytes)
    parser.add_argument('-q', '--quotechar', help="log quotecher", default=b"\"", type=bytes)
    parser.add_argument('-p', '--prefix', help="log collumn prefix", default=b"", type=bytes)
    parser.add_argument('filepath')
    args = parser.parse_args()
    args.delimiter = args.delimiter.replace(b"\\t", b"\t")
    validatedatetime(args)
    search(args)

if __name__=='__main__':
    main()





# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function, unicode_literals

import bitstring
def readbit(fio, bitpos, readbit=48):
    bytepos, bit = divmod(bitpos, 8)
    fio.seek(bytepos)
    data = bitstring.BitStream(bytes=fio.read(readbit / 8 + 1))
    data.read(bit)
    return data.read(readbit)
def getbits(fio):
    pass



# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function, unicode_literals

import os
from .. import bz2search
from .. import bz2bin
from .. import create_searcher

def test_searchblock():
    sample = os.path.join(os.path.dirname(__file__), 'sample/sample.bz2')
    with open(sample) as fd:
        pos = bz2search.searchblock(fd, 5)
        assert pos > 0
        print(pos)
        rd = bz2search.BlockReader(fd, pos)
        assert rd
        line = rd.next()
        assert line
        print(len(line))
        print(line)
        assert "\n"==line[-1]
        # next block
        fd = open(sample)
        fd.seek(0)
        pos = bz2search.searchblock(fd, (pos+80)//8)
        print('pos',pos)
        rd = None
        rd = bz2search.BlockReader(fd, pos)
        assert rd
        line = rd.next()
        assert line
        assert "\n"==line[-1]
        fd2 = open(sample)
        rd.reset(fd2, pos)
        assert rd
        line = rd.next()
        assert line
        assert "\n"==line[-1]

def test_binsearch():
    valtype = int
    sample = os.path.join(os.path.dirname(__file__), 'sample/sample.bz2')
    binsearch = create_searcher(sample, 0,
                                lower=333, upper=334, valtype=valtype)
    with binsearch as res:
        assert res
        results = list(res)
        print(results[:2])
        assert 2 == len(results)
        assert 333 <= int(results[0][0]) <= 334
        assert 333 <= int(results[1][0]) <= 334
        print(results[:2])






# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function, unicode_literals
import bz2
import os
import tempfile
from .. import create_searcher
from ..import binsearch
binsearch.DEBUG = 1
def test_TextBinSearch():
    sample = os.path.join(os.path.dirname(__file__), 'sample/sample.bz2')
    
    with bz2.BZ2File(sample) as fio:
        with tempfile.NamedTemporaryFile('w') as out:
            out.write(fio.read())
            out.seek(0)
            searcher =  create_searcher(out.name, 0, lower=333, upper=334, valtype=int)
            with searcher as res:
                results = list(res)
                print(results[:2])
                assert 2 == len(results)
                assert 3 == len(results[0][0])
                assert 333 <= int(results[0][0]) < 334






# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function, unicode_literals

import datetime
import os
import random
import bz2
from ..scripts import logdatetimesearch
from .. import binsearch

sample = os.path.join(os.path.dirname(__file__), 'sample/sample_datetime.bz2')

def generate(fromdate):
    if os.path.exists(sample):
        return sample

    bzf = bz2.BZ2File(sample, 'w', compresslevel=1)
    ptr = fromdate
    for i in range(100):
        for j in range(10):
            rows = [i*10 + j, ptr.strftime('%Y-%m-%d %H:%M:%S')] \
                + [random.randint(0, 100)]*10
            line = " ".join(map(bytes, rows)) + "\n"
            bzf.write(bytes(line))

        ptr += datetime.timedelta(minutes=random.randint(0, 10))

    return sample

def test_search():
    binsearch.DEBUG = 1
    results = []
    def mock(*string):
        #print(*string)
        results.append(string)

    fromdate = datetime.datetime(2015, 1, 20, 13, 0, 0)
    sample = generate(fromdate)
    logdatetimesearch.info = mock
    import sys
    sys.argv = ['logdatetimesearch.py', '-f', '2015-01-20 15:10:00', '-t', '2015-01-20 15:20:00', sample]
    logdatetimesearch.main()
    for row in results[1:10]:
        print(row)
        vals = row[0].split()
        vals = " ".join(vals[1:3])
        assert vals>='2015-01-20 15:10'
        assert vals<='2015-01-20 15:20'

    reload(logdatetimesearch)
    reload(sys)




# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function, unicode_literals
import os
from .binsearch import Reader, Comparator, TextBinSearch
from .bz2bin import BZ2BinSearch

def create_searcher(filepath, col, colupper=None, delimiter=b"\t", quotechar=b"\"", valtype=str, lower=None, upper=None, ext=None):
    if not os.path.exists(filepath):
        raise Exception('cannot found: {}'.format(filepath))

    if ext is None:
        base, ext = os.path.splitext(filepath)

    if ext=='.bz2':
        searcher = BZ2BinSearch
    else:
        searcher = TextBinSearch

    reader = Reader(col, colupper, delimiter, quotechar, valtype)
    comparator = Comparator(lower, upper)
    return searcher(filepath, comparator, reader)


